/******************************************************************************
*
* project name:    Scorch
* file name:       Main##.c
* initial date:    12.4.2004
* author:          mhubi@freesurf.ch
* description:     main c-source file
*
*******************************************************************************
*
* Scorch -- another tank-game
* Copyright (C) 2004  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#include "main-h.h"

#if defined(TI_BIG)
	#warning "Building the ti92+/v200 version"
#elif defined(TI_SMALL)
	#warning "Building the ti89 version"
#endif

#if defined(VTI)
	#warning "Compiling for VTI"
#endif

#if defined(TESTING)
	#warning "Test version"
#endif

//extern-variables
extern INT_HANDLER save_int_1;
extern INT_HANDLER save_int_5;
extern PLAYERINF player[5];
extern LCD_BUFFER homescreen;
extern GAME game;
extern int earth[240];
extern int curfunc;

// Main Function
void _main(void)
{
	
	game.status = NEWGAME;
	
	save_int_1 = GetIntVec (AUTO_INT_1);
	save_int_5 = GetIntVec (AUTO_INT_5);
	
	#if defined(VTI)
	
	if (IsVTI() == TRUE) {
		ST_helpMsg("Warning, certain parts are faster on VTI");
		ngetchx();
		ST_eraseHelp();
	}
	
	#endif
	
	LCD_save(homescreen);
	
	Intro();
	
	while (game.status == NEWGAME) {
		
		Welcome();
		
		if (game.status != GAMELOADED) {
		
			PlayerSettings();
			
			GameSettings();
		
		}
		
		GameLoop();
		
		//Waitkey();
		
		while (game.status != NEWGAME) {
			MainMenu();
		}
		
	}
	
}


void Intro(void) {
	
	unsigned int loop1, line, square, loop2, place, x;
	
	FastFillRect(LCD_MEM, 0, 0, CALC(159,239), CALC(99,127), A_XOR);
	
	FontSetSys(F_8x10);
	DrawStr(CALC(48,88), CALC(40,54), "hubisoft", A_REVERSE);
	FontSetSys(F_6x8);
	DrawStr(CALC(56,96), CALC(60,74), "presents", A_REVERSE);
	
	for (loop1 = 0; loop1 < 30; loop1++) {
		#if defined(VTI)
		VTIPoking();
		#else
		idle();
		#endif
	}
	
	for (loop1 = 0; loop1 < 4; loop1++) {
		place = loop1;
		for (line = 0; line < CALC(13,16); line++) {
			if (place > 3) {
				place = 0;
			}
			for (square = 0; square < CALC(20,30); square+=4) {
				x = square*8 + place * 8;
				if (x < 240) {
					ClipSprite8_AND_R(x, line*8 , 8, mask, LCD_MEM);
				}
			}
			place++;
		}
		for (loop2 = 0; loop2 < 7 && loop1 != 3; loop2++) {
			#if defined(VTI)
			VTIPoking();
			#else
			idle();
			#endif
		};
	}
	
	SetIntVec (AUTO_INT_1, DUMMY_HANDLER);
	SetIntVec (AUTO_INT_5, DUMMY_HANDLER);
	GrayOn();
	
	//s-c-o-r-c-h
	
}

void Welcome(void) {
	
	unsigned char title[CALC(770,1600)];
	
	curfunc = WELCOME;
	
	GrayClearScreen();
	
	//titleimage
	
	DecompRLEVert(title1_rle, title, CALC(770,1600), CALC(14,20), CALC(55,80));
	
	//#if defined(TI_BIG)
	SpriteX8_OR(CALC(24,30),0,CALC(55,80),title,CALC(14,20),DPLANE);
	
	DecompRLEVert(title2_rle, title, CALC(770,1600), CALC(14,20), CALC(55,80));
	
	SpriteX8_OR(CALC(24,30),0,CALC(55,80),title,CALC(14,20),LPLANE);
	//#endif
	
	//choose rounds and players
	P_R_choose();
	
}


void GameLoop(void) {
	
	//int key = 0;
	
	randomize();
	
	Wait();
	
	if (game.status == NEWGAME) {
		game.status = NEWROUND;
	}
	
	//main-game-loop	
	while (game.rounds_now <= game.rounds && game.status >= NORMAL) {
		
		curfunc = GAMELOOP;
		
		GrayClearScreen();
		
		if (game.status != GAMELOADED) {
			
			InitRound();
			
			Landscape(game.land_factor);
			
			game.pl_active = 0;
		
		}
		
		game.status = NORMAL;
		
		DrawScreen();
		
		PlayRound();
		
		// show the statistics
		Statistics();
		
		Waitkey();
		
		if (game.rounds_now == game.rounds) {
			game.status = ENDGAME;
			return;
		}
		
		
		// if there's a round to play show the shop for each human-player
		else if (game.rounds_now != game.rounds) {
			
			/*while ((key = KeyPressed()) != K_ENTER) {
				pokeIO(0x600005, 0b10111);
			}*/
			
			//curfunc = SHOP;
			
			for (game.pl_check = 0; game.pl_check < game.pl_num && game.status == NORMAL; game.pl_check++) {
				if (player[game.pl_check].type == 1 && player[game.pl_check].credits > 0) {
					
					// show the shop
					Shop(game.pl_check);
					
				}
				
				//#if defined (TESTING)
				else if (player[game.pl_check].type == 2) {
					AiShop();
				}
				//#endif
				
			}
			if (game.status == NORMAL) {
				game.status = NEWROUND;
				game.rounds_now++;
			}
		
		}
		
		
	} // main-game-loop-end
	
	
} // function-end


void PlayRound(void) {
	
	int key = 0;
	TARGET target;
	
	curfunc = PLAYROUND;
	
	// round-loop, each players round, exits only if status != NORMAL		
	//for (game.pl_active = 0; game.pl_active <= game.pl_num && game.status == NORMAL; game.pl_active++) {
	for (; game.pl_active <= game.pl_num && game.status == NORMAL; game.pl_active++) {
		
		// if each player has shot player no 1 can shoot
		if (game.pl_active == game.pl_num) {
			game.pl_active = 0;
		}
		
		//DrawScreen();
		
		if (_keytest(RR_APPS)) {
			game.status = QUIT;
			MainMenu();
		}
		else if (_keytest(RR_F2)) {
			game.status = FASTQUIT;
			LeadOut();
		}
		
		// he can only shoot if he's still alive
		if (player[game.pl_active].life != 0) {
			
			//Dispinf(game.pl_active);
			//GrayClearScreen();
			//DrawScreen();
			
			// case 1: human-player
			if (player[game.pl_active].type == 1) {
				
				key = 0;
				
				Dispinf(game.pl_active);
				
				//Aidata();
				
				// !! possible to change the status --> everything: status == NORMAL
				while (key != K_ENTER && game.status == NORMAL) {
					
					while (!(key = KeyPressed()) && game.status == NORMAL) {
						pokeIO(0x600005, 0b10111);
					}
					
					//choose angle
					if (key == K_LEFT || key == K_LEFT + K_2ND || key == K_RIGHT || key == K_RIGHT + K_2ND) {
						Angle(game.pl_active, key);
						Delay(AUTO_REPEAT);
					}
					
					else if (key == K_UP || key == K_UP + K_2ND || key == K_DOWN || key == K_DOWN + K_2ND) {
						Power(&player[game.pl_active].power, key);
						Delay(AUTO_REPEAT);
					}
					
					else if (key == K_Q) {
						Inventory(game.pl_active);
						Dispinf(game.pl_active);
					}
					
				}
				
				if (key == K_ENTER && game.status == NORMAL) {
					
					MainShot(game.pl_active, &target);
					
					/*if (player[game.pl_active].bomb[0] < LASERLITTLE) {
						ShotEngine(game.pl_active, &target);
					}
					Explosion(target.x, target.y, game.pl_active);*/
					
				}
				
				
			}
			
			// case 2: computer-player
			if (player[game.pl_active].type == 2 && game.status == NORMAL) {
				
				//#if defined(TESTING)
				
				AiMain(&target, PRE);
				
				Dispinf(game.pl_active);
				
				//#endif
				
				//Inventory(game.pl_active);
				
				MainShot(game.pl_active, &target);
				
				/*if (player[game.pl_active].bomb[0] < LASERLITTLE) {
					ShotEngine(game.pl_active, &target);
				}
				Explosion(target.x, target.y, game.pl_active);*/
				
				//oldold
				
			}
			
			// after each shot control if a player died with the last shot
			// --> credits...
			for (game.pl_check = 0; game.pl_check < game.pl_num && game.status == NORMAL; game.pl_check++) {
				if (player[game.pl_check].life != 0) {
					//Health_control(game.pl_check);
					//Tank_fall(game.pl_check);
					
					if (player[game.pl_check].health <= 0 && player[game.pl_check].life != 0) {
						
						player[game.pl_check].bomb[0] = 0;
						Explosion(player[game.pl_check].x, player[game.pl_check].y - 4, game.pl_check);
						
						game.pl_alive--;
						player[game.pl_check].life = 0;
						
						if (player[game.pl_active].life != 0 && player[game.pl_active].credits < 59000) {
							player[game.pl_active].credits += 1000;
						}
					
					}
				
				}
				
			}
			
			if (game.status == NEWROUND) {
				game.status = NORMAL;
			}
			
			// get out of the round-loop if less than two players are alive
			// --> statistics
			if (game.pl_alive < 2) { 
				
				return;
					
			} // end of "end-of-round"-loop
			
			// ai if computer-player is active
			if (player[game.pl_active].type == 2 && game.status == NORMAL) {
				//#if defined(TESTING)
				AiMain(&target, AFTER);
				/*#else
				Ai();
				#endif*/
			}
			
		} // end of the shot-n-control-procedure
		
		
		
	} // round-loop-end
	
	
}


void Extro(void) {
	
	
	// shows the end-screen with the quotation
	GrayClearScreen();
	FontSetSys(CALC(F_6x8,F_8x10));
	GrayDrawStr(CALC(20,30),CALC(25,35),(char*)"There's many a boy",A_NORMAL);
	GrayDrawStr(CALC(20,30),CALC(34,46),(char*)"here today who looks",A_NORMAL);
	GrayDrawStr(CALC(20,30),CALC(43,57),(char*)"on war as all glory,",A_NORMAL);
	GrayDrawStr(CALC(20,30),CALC(52,68),(char*)"but, boys, it is all",A_NORMAL);
	GrayDrawStr(CALC(20,30),CALC(61,79),(char*)"hell. - Gen. Sherman",A_NORMAL);
	
	GrayDrawStrExt(0,CALC(80,100),(char*)"hubisoft 05 -- mhubi@freesurf.ch",A_NORMAL | A_CENTERED, CALC(F_4x6,F_6x8));
	
	Waitkey();
	
}


void Delay(unsigned int delay) {
	
	unsigned int loop1;
	
	for (loop1 = 0; loop1 <= delay; loop1++) {
		#if defined(VTI)
		VTIPoking();
		#else
		pokeIO(0x600005, 0b10111);
		#endif
	}
	
}


void LeadOut(void) {
	
	if (game.status != FASTQUIT) {
		Extro();
	}
	
	Wait();
	
	GrayOff();
	SetIntVec (AUTO_INT_1, save_int_1);
	SetIntVec (AUTO_INT_5, save_int_5);
	
	LCD_restore(homescreen);
	
	if (game.status != FASTQUIT) {
		ST_helpMsg(CALC("      ~~ Scorch "COMMENT_VERSION_STRING"   by hubisoft ~~",
										"                               ~~ Scorch "COMMENT_VERSION_STRING"   by hubisoft ~~"));
	}
	
	exit(0);
	
}

void MainMenu(void) {
	
	int key = 0;
	LCD_BUFFER plane1, plane2;
	int curfunc_old;
	short success = 0;
	
	Wait();
	
	FastCopyScreen_R(LPLANE, plane1);
	FastCopyScreen_R(DPLANE, plane2);
	
	curfunc_old = curfunc;
	curfunc = MAINMENU;
	
	Rectangle();
	
	GrayDrawStrExt(0, CALC(15,29), (char*)"NEXT ACTION?", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
	
	GrayDrawStrExt2B(0, CALC(28,42), (char*)"Return    = [1]", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
	
	GrayDrawStrExt2B(0, CALC(38,52), (char*)"Quit Game = [2]", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
	
	GrayDrawStrExt2B(0, CALC(48,62), (char*)"Mass Kill = [3]", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
	
	GrayDrawStrExt2B(0, CALC(58,72), (char*)"New Game  = [4]", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
	
	GrayDrawStrExt2B(0, CALC(68,82), (char*)"Save Game = [5]", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
	
	GrayDrawStrExt2B(0, CALC(78,92), (char*)"Load Game = [6]", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
	
	if (game.status == ENDGAME) {
		DeleteMenuEntry(1);
	}
	if (curfunc_old != PLAYROUND) {
		DeleteMenuEntry(3);
	}
	if (game.status != ENDGAME) {
		DeleteMenuEntry(4);
	}
	if (curfunc_old != PLAYROUND) {
		DeleteMenuEntry(5);
	}
	if (curfunc_old != WELCOME && curfunc_old != PLAYROUND) {
		DeleteMenuEntry(6);
	}
	
	
	
	//Memory();
	
	while ((key < K_1 || key > K_6) && key != K_ESC) {
		
		key = KeyPressed();
		
		pokeIO(0x600005, 0b10111);
		
	}
	
	if (key == K_2 || (key == K_ESC && game.status == ENDGAME)) {
		game.status = QUIT;
		LeadOut();
	}
	
	
	if (key == K_5 && curfunc_old == PLAYROUND) {
		SaveGame();
	}
	else if (key == K_6 && (curfunc_old == PLAYROUND || curfunc_old == WELCOME)) {
		success = LoadGame();
		
		if (curfunc_old == WELCOME) {
			game.status = GAMELOADED;
			curfunc = curfunc_old;
			return;
		}
		
	}
	else if (key == K_4 && game.status == ENDGAME) {
		game.status = NEWGAME;
		curfunc = curfunc_old;
		return;
	}
	
	
	if (key == K_6 && success == TRUE && curfunc_old != WELCOME) {
		GrayClearScreen();
		game.status = NORMAL;
		DrawScreen();
	}
	else if (game.status != ENDGAME) {
		game.status = NORMAL;
		GrayClearScreen();
		
		FastCopyScreen_R(plane1, LPLANE);
		FastCopyScreen_R(plane2, DPLANE);
	}
	
	if (key == K_3 && curfunc_old == PLAYROUND) {
		game.status = NEWROUND;
		MassKill();
	}
	
	curfunc = curfunc_old;
	
	Wait();
	
	
}

//prototype from the tigcc-tools-suite/ttmem
void Memory(void) {
	
	char string[100];
	
	FontSetSys(F_4x6);
	HeapCompress();
  sprintf(string, "avail=%lu bytes (maxblock=%lu)", HeapAvail(), HeapMax());
  GrayDrawStr(32, 70, string, A_NORMAL);
	
}


#if defined(VTI)

// VTI detection routine (thanx go to Julien Muchembled); taken from travis fischers icehockey68k
short IsVTI(void);
asm("
IsVTI:
    trap   #12         /* enter supervisor mode. returns old (%sr) in %d0.w   */
    move.w #0x3000,%sr /* set a non-existing flag in %sr (but keep s-flag !!) */
    swap   %d0         /* save %d0.w content in upper part of %d0             */
    move.w %sr,%d0     /* get %sr content and check for non-existing flag     */
    btst   #12,%d0     /* this non-existing flag can only be set on the VTI   */
    bne    __VTI           
    swap   %d0         /* restore old %sr content and return 0                */
    move.w %d0,%sr
    moveq  #0,%d0
    rts
__VTI:
    swap   %d0         /* restore old %sr content and return 1                */
    move.w %d0,%sr
    moveq  #1,%d0
    rts
");

void VTIPoking(void) {
	
	unsigned int loop1;
	
	for (loop1 = 0; loop1 < 1430; loop1++);
	
}

#endif

#if defined(TESTING)

void Debugging(unsigned int num) {
	
	GrayDrawStr(40, 80, player[num].name, A_NORMAL);
  GraySprite16_XOR(100,80,5,player[num].tank,player[num].tank,LPLANE,DPLANE);
  Waitkey();
	
}

#endif

//ti89 version: completely done